@@ -206,7 +206,7 @@ class AuthHandler (object):
             event.wait(0.1)
             if not self.transport.is_active():
                 e = self.transport.get_exception()
-                if (e is None) or issubclass(e.__class__, EOFError):
+                if e is None or isinstance(e, EOFError):
                     e = AuthenticationException('Authentication failed.')
                 raise e
             if event.is_set():
@@ -218,9 +218,7 @@ class AuthHandler (object):
             e = self.transport.get_exception()
             if e is None:
                 e = AuthenticationException('Authentication failed.')
-            # this is horrible.  Python Exception isn't yet descended from
-            # object, so type(e) won't work. :(
-            if issubclass(e.__class__, PartialAuthentication):
+            if isinstance(e, PartialAuthentication):
                 return e.allowed_types
             raise e
         return []
@@ -433,7 +431,7 @@ Error Message: {}
                 (self.auth_username != username)):
             self._log(
                 WARNING,
-                'Auth rejected because the client attempted to change username in mid-flight'  # noqa
+                'Auth rejected because the client attempted to change username in mid-flight'
             )
             self._disconnect_no_more_auth()
             return
@@ -480,7 +478,7 @@ Error Message: {}
                     'Auth rejected: public key: {}'.format(str(e)))
                 key = None
             except Exception as e:
-                msg = 'Auth rejected: unsupported or mangled public key ({}: {})' # noqa
+                msg = 'Auth rejected: unsupported or mangled public key ({}: {})'
                 self._log(INFO, msg.format(e.__class__.__name__, e))
                 key = None
             if key is None:
