@@ -26,7 +26,6 @@ import inspect
 import os
 import socket
 import warnings
-from errno import ECONNREFUSED, EHOSTUNREACH
 
 from paramiko.agent import Agent
 from paramiko.common import DEBUG
@@ -37,9 +36,7 @@ from paramiko.ed25519key import Ed25519Key
 from paramiko.hostkeys import HostKeys
 from paramiko.py3compat import string_types
 from paramiko.rsakey import RSAKey
-from paramiko.ssh_exception import (
-    SSHException, BadHostKeyException, NoValidConnectionsError
-)
+from paramiko.ssh_exception import SSHException, BadHostKeyException, BadAuthenticationType
 from paramiko.transport import Transport
 from paramiko.util import retry_on_signal, ClosingContextManager
 
@@ -187,29 +184,6 @@ class SSHClient (ClosingContextManager):
             policy = policy()
         self._policy = policy
 
-    def _families_and_addresses(self, hostname, port):
-        """
-        Yield pairs of address families and addresses to try for connecting.
-
-        :param str hostname: the server to connect to
-        :param int port: the server port to connect to
-        :returns: Yields an iterable of ``(family, address)`` tuples
-        """
-        guess = True
-        addrinfos = socket.getaddrinfo(
-            hostname, port, socket.AF_UNSPEC, socket.SOCK_STREAM)
-        for (family, socktype, proto, canonname, sockaddr) in addrinfos:
-            if socktype == socket.SOCK_STREAM:
-                yield family, sockaddr
-                guess = False
-
-        # some OS like AIX don't indicate SOCK_STREAM support, so just
-        # guess. :(  We only do this if we did not get a single result marked
-        # as socktype == SOCK_STREAM.
-        if guess:
-            for family, _, _, _, sockaddr in addrinfos:
-                yield family, sockaddr
-
     def connect(
         self,
         hostname,
@@ -252,8 +226,8 @@ class SSHClient (ClosingContextManager):
                 this to occur - *just* the certificate.)
 
             - Any key we can find through an SSH agent
-            - Any "id_rsa", "id_dsa" or "id_ecdsa" key discoverable in
-              ``~/.ssh/``
+            - Any "id_rsa", "id_dsa", "id_ecdsa", or "id_ed25519" key
+              discoverable in ``~/.ssh/``
 
               - When OpenSSH-style public certificates exist that match an
                 existing such private key (so e.g. one has ``id_rsa`` and
@@ -324,37 +298,9 @@ class SSHClient (ClosingContextManager):
             Added the ``passphrase`` argument.
         """
         if not sock:
-            errors = {}
-            # Try multiple possible address families (e.g. IPv4 vs IPv6)
-            to_try = list(self._families_and_addresses(hostname, port))
-            for af, addr in to_try:
-                try:
-                    sock = socket.socket(af, socket.SOCK_STREAM)
-                    if timeout is not None:
-                        try:
-                            sock.settimeout(timeout)
-                        except:
-                            pass
-                    retry_on_signal(lambda: sock.connect(addr))
-                    # Break out of the loop on success
-                    break
-                except socket.error as e:
-                    # Raise anything that isn't a straight up connection error
-                    # (such as a resolution error)
-                    if e.errno not in (ECONNREFUSED, EHOSTUNREACH):
-                        raise
-                    # Capture anything else so we know how the run looks once
-                    # iteration is complete. Retain info about which attempt
-                    # this was.
-                    errors[addr] = e
-
-            # Make sure we explode usefully if no address family attempts
-            # succeeded. We've no way of knowing which error is the "right"
-            # one, so we construct a hybrid exception containing all the real
-            # ones, of a subclass that client code should still be watching for
-            # (socket.error)
-            if len(errors) == len(to_try):
-                raise NoValidConnectionsError(errors)
+            sock = retry_on_signal(
+                lambda: socket.create_connection((hostname, port), timeout)
+            )
 
         t = self._transport = Transport(
             sock, gss_kex=gss_kex, gss_deleg_creds=gss_deleg_creds
@@ -514,6 +460,8 @@ class SSHClient (ClosingContextManager):
         :raises: `.SSHException` -- if the server fails to invoke a shell
         """
         chan = self._transport.open_session()
+        if environment:
+            chan.update_environment(environment)
         chan.get_pty(term, width, height, width_pixels, height_pixels)
         chan.invoke_shell()
         return chan
@@ -614,7 +562,20 @@ class SSHClient (ClosingContextManager):
             except Exception as e:
                 saved_exception = e
 
-        if pkey is not None:
+        # detect what authentication methods the server supports
+        # mirrors what openssh client does
+        try:
+            self._transport.auth_none(username)
+        except BadAuthenticationType as e:
+            allowed_types = set(e.allowed_types)
+        else:
+            return  # successful login with no auth
+
+        # some servers do not return allowed auth methods
+        if not allowed_types:
+            allowed_types = {'password', 'publickey'}
+
+        if pkey is not None and 'publickey' in allowed_types:
             try:
                 self._log(
                     DEBUG,
@@ -628,7 +589,7 @@ class SSHClient (ClosingContextManager):
             except SSHException as e:
                 saved_exception = e
 
-        if not two_factor:
+        if not two_factor and 'publickey' in allowed_types:
             for key_filename in key_filenames:
                 for pkey_class in (RSAKey, DSSKey, ECDSAKey, Ed25519Key):
                     try:
@@ -644,7 +605,7 @@ class SSHClient (ClosingContextManager):
                     except SSHException as e:
                         saved_exception = e
 
-        if not two_factor and allow_agent:
+        if allow_agent and not two_factor and 'publickey' in allowed_types:
             if self._agent is None:
                 self._agent = Agent()
 
@@ -652,8 +613,6 @@ class SSHClient (ClosingContextManager):
                 try:
                     id_ = hexlify(key.get_fingerprint())
                     self._log(DEBUG, 'Trying SSH agent key {}'.format(id_))
-                    # for 2-factor auth a successfully auth'd key password
-                    # will return an allowed 2fac auth method
                     allowed_types = set(
                         self._transport.auth_publickey(username, key))
                     two_factor = (allowed_types & two_factor_types)
@@ -663,9 +622,8 @@ class SSHClient (ClosingContextManager):
                 except SSHException as e:
                     saved_exception = e
 
-        if not two_factor:
+        if look_for_keys and not two_factor and 'publickey' in allowed_types:
             keyfiles = []
-
             for keytype, name in [
                 (RSAKey, "rsa"),
                 (DSSKey, "dsa"),
@@ -683,16 +641,11 @@ class SSHClient (ClosingContextManager):
                         if os.path.isfile(full_path + '-cert.pub'):
                             keyfiles.append((keytype, full_path + '-cert.pub'))
 
-            if not look_for_keys:
-                keyfiles = []
-
             for pkey_class, filename in keyfiles:
                 try:
                     key = self._key_from_filepath(
                         filename, pkey_class, passphrase,
                     )
-                    # for 2-factor auth a successfully auth'd key will result
-                    # in ['password']
                     allowed_types = set(
                         self._transport.auth_publickey(username, key))
                     two_factor = (allowed_types & two_factor_types)
@@ -702,14 +655,23 @@ class SSHClient (ClosingContextManager):
                 except (SSHException, IOError) as e:
                     saved_exception = e
 
-        if password is not None:
+        # possible two_factor second factors
+        # (allowed_types could have been updated, only if two_factor)
+
+        if password is not None and 'password' in allowed_types:
             try:
-                self._transport.auth_password(username, password)
-                return
+                self._log(DEBUG, 'Trying password')
+                allowed_types = set(
+                    self._transport.auth_password(username, password))
+                two_factor = allowed_types & two_factor_types
+                if not two_factor:
+                    return
             except SSHException as e:
                 saved_exception = e
-        elif two_factor:
+
+        if 'keyboard-interactive' in allowed_types:
             try:
+                self._log(DEBUG, 'Trying interactive auth')
                 self._transport.auth_interactive_dumb(username)
                 return
             except SSHException as e:
