@@ -15,6 +15,8 @@
 # You should have received a copy of the GNU Lesser General Public License
 # along with Paramiko; if not, write to the Free Software Foundation, Inc.,
 # 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
+
+import socket
 from io import BytesIO
 
 from paramiko.common import (
@@ -187,9 +189,9 @@ class BufferedFile (ClosingContextManager):
             raise IOError('File is not open for reading')
         if (size is None) or (size < 0):
             # go for broke
-            result = self._rbuffer
+            chunks = [self._rbuffer]
+            self._pos += len(self._rbuffer)
             self._rbuffer = bytes()
-            self._pos += len(result)
             while True:
                 try:
                     new_data = self._read(self._DEFAULT_BUFSIZE)
@@ -197,10 +199,10 @@ class BufferedFile (ClosingContextManager):
                     new_data = None
                 if (new_data is None) or (len(new_data) == 0):
                     break
-                result += new_data
+                chunks.append(new_data)
                 self._realpos += len(new_data)
                 self._pos += len(new_data)
-            return result
+            return bytes().join(chunks)
         if size <= len(self._rbuffer):
             result = self._rbuffer[:size]
             self._rbuffer = self._rbuffer[size:]
@@ -288,6 +290,9 @@ class BufferedFile (ClosingContextManager):
                 break
             try:
                 new_data = self._read(n)
+            except socket.timeout:
+                self._rbuffer = line
+                raise
             except EOFError:
                 new_data = None
             if (new_data is None) or (len(new_data) == 0):
