@@ -261,7 +261,6 @@ class SFTPClient(BaseSFTP, ClosingContextManager):
                     num = self._async_request(type(None), CMD_READDIR, handle)
                     nums.append(num)
 
-
                 # For each of our sent requests
                 # Read and parse the corresponding packets
                 # If we're at the end of our queued requests, then fire off
@@ -291,7 +290,6 @@ class SFTPClient(BaseSFTP, ClosingContextManager):
                 self._request(CMD_CLOSE, handle)
                 return
 
-
     def open(self, filename, mode='r', bufsize=-1):
         """
         Open a file on the remote server.  The arguments are the same as for
@@ -686,7 +684,15 @@ class SFTPClient(BaseSFTP, ClosingContextManager):
                 reader=fl, writer=fr, file_size=file_size, callback=callback
             )
         if confirm:
-            s = self.stat(remotepath)
+            try:
+                s = self.stat(remotepath)
+            except IOError as e:
+                if e.errno == SFTP_NO_SUCH_FILE:
+                    raise IOError(SFTP_NO_SUCH_FILE,
+                        "STAT failed after successful PUT,"
+                        " possibly because the server deleted or moved the file."
+                        " STAT is skipped if confirm=false. Original error: " + str(e))
+                raise
             if s.st_size != size:
                 raise IOError(
                     'size mismatch in put!  {} != {}'.format(s.st_size, size))
@@ -832,9 +838,8 @@ class SFTPClient(BaseSFTP, ClosingContextManager):
                     self._convert_status(msg)
                 return t, msg
 
-            # can not rewrite this to deal with E721, either as a None check
-            # nor as not an instance of None or NoneType
-            if fileobj is not type(None):  # noqa
+            # can not rewrite this to deal with E721
+            if fileobj is not type(None):  # noqa: E721
                 fileobj._async_response(t, msg, num)
             if waitfor is None:
                 # just doing a single check
