@@ -20,11 +20,8 @@
 Some unit tests for SSHClient.
 """
 
-from __future__ import with_statement, print_function
-
 import gc
 import os
-import platform
 import socket
 import threading
 import time
@@ -224,13 +221,13 @@ class ClientTest(unittest.TestCase):
 
 
 class SSHClientTest(ClientTest):
-    def test_1_client(self):
+    def test_client(self):
         """
         verify that the SSHClient stuff works too.
         """
         self._test_connection(password='pygmalion')
 
-    def test_2_client_dsa(self):
+    def test_client_dsa(self):
         """
         verify that SSHClient works with a DSA key.
         """
@@ -242,7 +239,7 @@ class SSHClientTest(ClientTest):
         """
         self._test_connection(key_filename=_support('test_rsa.key'))
 
-    def test_2_5_client_ecdsa(self):
+    def test_client_ecdsa(self):
         """
         verify that SSHClient works with an ECDSA key.
         """
@@ -252,7 +249,7 @@ class SSHClientTest(ClientTest):
     def test_client_ed25519(self):
         self._test_connection(key_filename=_support('test_ed25519.key'))
 
-    def test_3_multiple_key_files(self):
+    def test_multiple_key_files(self):
         """
         verify that SSHClient accepts and tries multiple key files.
         """
@@ -289,7 +286,8 @@ class SSHClientTest(ClientTest):
         """
         # Until #387 is fixed we have to catch a high-up exception since
         # various platforms trigger different errors here >_<
-        self.assertRaises(SSHException,
+        self.assertRaises(
+            SSHException,
             self._test_connection,
             key_filename=[_support('test_rsa.key')],
             allowed_keys=['ecdsa-sha2-nistp256'],
@@ -335,7 +333,7 @@ class SSHClientTest(ClientTest):
         # code path (!) so we're punting too, sob.
         pass
 
-    def test_4_auto_add_policy(self):
+    def test_auto_add_policy(self):
         """
         verify that SSHClient's AutoAddPolicy works.
         """
@@ -358,7 +356,7 @@ class SSHClientTest(ClientTest):
         new_host_key = list(self.tc.get_host_keys()[hostname].values())[0]
         self.assertEqual(public_host_key, new_host_key)
 
-    def test_5_save_host_keys(self):
+    def test_save_host_keys(self):
         """
         verify that SSHClient correctly saves a known_hosts file.
         """
@@ -385,15 +383,11 @@ class SSHClientTest(ClientTest):
 
         os.unlink(localname)
 
-    def test_6_cleanup(self):
+    def test_cleanup(self):
         """
         verify that when an SSHClient is collected, its transport (and the
         transport's packetizer) is closed.
         """
-        # Skipped on PyPy because it fails on travis for unknown reasons
-        if platform.python_implementation() == "PyPy":
-            return
-
         threading.Thread(target=self._run).start()
 
         self.tc = paramiko.SSHClient()
@@ -438,7 +432,7 @@ class SSHClientTest(ClientTest):
 
         self.assertTrue(self.tc._transport is None)
 
-    def test_7_banner_timeout(self):
+    def test_banner_timeout(self):
         """
         verify that the SSHClient has a configurable banner timeout.
         """
@@ -457,7 +451,7 @@ class SSHClientTest(ClientTest):
             **kwargs
         )
 
-    def test_8_auth_trickledown(self):
+    def test_auth_trickledown(self):
         """
         Failed key auth doesn't prevent subsequent pw auth from succeeding
         """
@@ -478,7 +472,7 @@ class SSHClientTest(ClientTest):
         self._test_connection(**kwargs)
 
     @slow
-    def test_9_auth_timeout(self):
+    def test_auth_timeout(self):
         """
         verify that the SSHClient has a configurable auth timeout
         """
@@ -491,7 +485,7 @@ class SSHClientTest(ClientTest):
         )
 
     @requires_gss_auth
-    def test_10_auth_trickledown_gsskex(self):
+    def test_auth_trickledown_gsskex(self):
         """
         Failed gssapi-keyex auth doesn't prevent subsequent key auth from succeeding
         """
@@ -502,7 +496,7 @@ class SSHClientTest(ClientTest):
         self._test_connection(**kwargs)
 
     @requires_gss_auth
-    def test_11_auth_trickledown_gssauth(self):
+    def test_auth_trickledown_gssauth(self):
         """
         Failed gssapi-with-mic auth doesn't prevent subsequent key auth from succeeding
         """
@@ -512,7 +506,7 @@ class SSHClientTest(ClientTest):
         )
         self._test_connection(**kwargs)
 
-    def test_12_reject_policy(self):
+    def test_reject_policy(self):
         """
         verify that SSHClient's RejectPolicy works.
         """
@@ -528,7 +522,7 @@ class SSHClientTest(ClientTest):
         )
 
     @requires_gss_auth
-    def test_13_reject_policy_gsskex(self):
+    def test_reject_policy_gsskex(self):
         """
         verify that SSHClient's RejectPolicy works,
         even if gssapi-keyex was enabled but not used.
@@ -674,16 +668,14 @@ class PasswordPassphraseTests(ClientTest):
             passphrase='television',
         )
 
-    def test_password_kwarg_used_for_passphrase_when_no_passphrase_kwarg_given(self): # noqa
+    def test_password_used_for_passphrase_when_no_passphrase(self):
         # Backwards compatibility: passphrase in the password field.
         self._test_connection(
             key_filename=_support('test_rsa_password.key'),
             password='television',
         )
 
-    def test_password_kwarg_not_used_for_passphrase_when_passphrase_kwarg_given(self): # noqa
-        # Sanity: if we're given both fields, the password field is NOT used as
-        # a passphrase.
+    def test_password_not_used_for_passphrase_when_passphrase_given(self):
         self.assertRaises(
             AuthenticationException,
             self._test_connection,
