@@ -20,8 +20,6 @@
 Some unit tests for the ssh2 protocol in Transport.
 """
 
-from __future__ import with_statement
-
 from binascii import hexlify
 import select
 import socket
@@ -32,8 +30,8 @@ import unittest
 from mock import Mock
 
 from paramiko import (
-    Transport, SecurityOptions, ServerInterface, RSAKey, DSSKey, SSHException,
-    ChannelException, Packetizer, AuthHandler,
+    Transport, SecurityOptions, ServerInterface, RSAKey, SSHException,
+    ChannelException, Packetizer, AuthHandler, BadHostKeyException
 )
 from paramiko import AUTH_FAILED, AUTH_SUCCESSFUL
 from paramiko import OPEN_SUCCEEDED, OPEN_FAILED_ADMINISTRATIVELY_PROHIBITED
@@ -69,9 +67,6 @@ Maybe.
 
 
 class NullServer (ServerInterface):
-    paranoid_did_password = False
-    paranoid_did_public_key = False
-    paranoid_key = DSSKey.from_private_key_file(_support('test_dss.key'))
 
     def get_allowed_auths(self, username):
         if username == 'slowdive':
@@ -168,7 +163,7 @@ class TransportTest(unittest.TestCase):
         self.assertTrue(event.is_set())
         self.assertTrue(self.ts.is_active())
 
-    def test_1_security_options(self):
+    def test_security_options(self):
         o = self.tc.get_security_options()
         self.assertEqual(type(o), SecurityOptions)
         self.assertTrue(('aes256-cbc', 'blowfish-cbc') != o.ciphers)
@@ -185,7 +180,7 @@ class TransportTest(unittest.TestCase):
         except TypeError:
             pass
 
-    def test_1b_security_options_reset(self):
+    def test_security_options_reset(self):
         o = self.tc.get_security_options()
         # should not throw any exceptions
         o.ciphers = o.ciphers
@@ -194,7 +189,7 @@ class TransportTest(unittest.TestCase):
         o.kex = o.kex
         o.compression = o.compression
 
-    def test_2_compute_key(self):
+    def test_compute_key(self):
         self.tc.K = 123281095979686581523377256114209720774539068973101330872763622971399429481072519713536292772709507296759612401802191955568143056534122385270077606457721553469730659233569339356140085284052436697480759510519672848743794433460113118986816826624865291116513647975790797391795651716378444844877749505443714557929  # noqa: E501
         self.tc.H = b'\x0C\x83\x07\xCD\xE6\x85\x6F\xF3\x0B\xA9\x36\x84\xEB\x0F\x04\xC2\x52\x0E\x9E\xD3'  # noqa: E501
         self.tc.session_id = self.tc.H
@@ -202,7 +197,7 @@ class TransportTest(unittest.TestCase):
         self.assertEqual(b'207E66594CA87C44ECCBA3B3CD39FDDB378E6FDB0F97C54B2AA0CFBF900CD995',
                          hexlify(key).upper())
 
-    def test_3_simple(self):
+    def test_simple(self):
         """
         verify that we can establish an ssh link with ourselves across the
         loopback sockets.  this is hardly "simple" but it's simpler than the
@@ -229,7 +224,7 @@ class TransportTest(unittest.TestCase):
         self.assertEqual(True, self.tc.is_authenticated())
         self.assertEqual(True, self.ts.is_authenticated())
 
-    def test_3a_long_banner(self):
+    def test_long_banner(self):
         """
         verify that a long banner doesn't mess up the handshake.
         """
@@ -247,7 +242,21 @@ class TransportTest(unittest.TestCase):
         self.assertTrue(event.is_set())
         self.assertTrue(self.ts.is_active())
 
-    def test_4_special(self):
+    def test_bad_hostkey(self):
+        badkey = RSAKey.from_private_key_file(_support('test_rsa.key'))
+        public_badkey = RSAKey(data=badkey.asbytes())
+        host_key = RSAKey.from_private_key_file(_support('test_rsa_2k_o.key'),
+                                                password='television')
+        self.ts.add_server_key(host_key)
+        event = threading.Event()
+        server = NullServer()
+        self.ts.start_server(event, server)
+        self.assertRaises(
+            BadHostKeyException, self.tc.connect,
+            hostkey=public_badkey, username='slowdive', password='pygmalion'
+        )
+
+    def test_special(self):
         """
         verify that the client can demand odd handshake settings, and can
         renegotiate keys in mid-stream.
@@ -266,7 +275,7 @@ class TransportTest(unittest.TestCase):
         self.ts.send_ignore(1024)
 
     @slow
-    def test_5_keepalive(self):
+    def test_keepalive(self):
         """
         verify that the keepalive will be sent.
         """
@@ -276,7 +285,7 @@ class TransportTest(unittest.TestCase):
         time.sleep(2)
         self.assertEqual('keepalive@lag.net', self.server._global_request)
 
-    def test_6_exec_command(self):
+    def test_exec_command(self):
         """
         verify that exec_command() does something reasonable.
         """
@@ -318,7 +327,7 @@ class TransportTest(unittest.TestCase):
         self.assertEqual('This is on stderr.\n', f.readline())
         self.assertEqual('', f.readline())
 
-    def test_6a_channel_can_be_used_as_context_manager(self):
+    def test_channel_can_be_used_as_context_manager(self):
         """
         verify that exec_command() does something reasonable.
         """
@@ -334,7 +343,7 @@ class TransportTest(unittest.TestCase):
                 self.assertEqual('Hello there.\n', f.readline())
                 self.assertEqual('', f.readline())
 
-    def test_7_invoke_shell(self):
+    def test_invoke_shell(self):
         """
         verify that invoke_shell() does something reasonable.
         """
@@ -348,7 +357,7 @@ class TransportTest(unittest.TestCase):
         chan.close()
         self.assertEqual('', f.readline())
 
-    def test_8_channel_exception(self):
+    def test_channel_exception(self):
         """
         verify that ChannelException is thrown for a bad open-channel request.
         """
@@ -359,7 +368,7 @@ class TransportTest(unittest.TestCase):
         except ChannelException as e:
             self.assertTrue(e.code == OPEN_FAILED_ADMINISTRATIVELY_PROHIBITED)
 
-    def test_9_exit_status(self):
+    def test_exit_status(self):
         """
         verify that get_exit_status() works.
         """
@@ -388,7 +397,7 @@ class TransportTest(unittest.TestCase):
         self.assertEqual(23, chan.recv_exit_status())
         chan.close()
 
-    def test_A_select(self):
+    def test_select(self):
         """
         verify that select() on a channel works.
         """
@@ -443,7 +452,7 @@ class TransportTest(unittest.TestCase):
         # ...and now is closed.
         self.assertEqual(True, p._closed)
 
-    def test_B_renegotiate(self):
+    def test_renegotiate(self):
         """
         verify that a transport can correctly renegotiate mid-stream.
         """
@@ -467,7 +476,7 @@ class TransportTest(unittest.TestCase):
 
         schan.close()
 
-    def test_C_compression(self):
+    def test_compression(self):
         """
         verify that zlib compression is basically working.
         """
@@ -490,7 +499,7 @@ class TransportTest(unittest.TestCase):
         chan.close()
         schan.close()
 
-    def test_D_x11(self):
+    def test_x11(self):
         """
         verify that an x11 port can be requested and opened.
         """
@@ -526,7 +535,7 @@ class TransportTest(unittest.TestCase):
         chan.close()
         schan.close()
 
-    def test_E_reverse_port_forwarding(self):
+    def test_reverse_port_forwarding(self):
         """
         verify that a client can ask the server to open a reverse port for
         forwarding.
@@ -563,7 +572,7 @@ class TransportTest(unittest.TestCase):
         self.tc.cancel_port_forward('127.0.0.1', port)
         self.assertTrue(self.server._listen is None)
 
-    def test_F_port_forwarding(self):
+    def test_port_forwarding(self):
         """
         verify that a client can forward new connections from a locally-
         forwarded port.
@@ -593,7 +602,7 @@ class TransportTest(unittest.TestCase):
         self.assertEqual(b'Hello!\n', cs.recv(7))
         cs.close()
 
-    def test_G_stderr_select(self):
+    def test_stderr_select(self):
         """
         verify that select() on a channel works even if only stderr is
         receiving data.
@@ -632,7 +641,7 @@ class TransportTest(unittest.TestCase):
         schan.close()
         chan.close()
 
-    def test_H_send_ready(self):
+    def test_send_ready(self):
         """
         verify that send_ready() indicates when a send would not block.
         """
@@ -656,7 +665,7 @@ class TransportTest(unittest.TestCase):
         chan.close()
         self.assertEqual(chan.send_ready(), True)
 
-    def test_I_rekey_deadlock(self):
+    def test_rekey_deadlock(self):
         """
         Regression test for deadlock when in-transit messages are received after MSG_KEXINIT sent
 
@@ -812,7 +821,7 @@ class TransportTest(unittest.TestCase):
         schan.close()
         chan.close()
 
-    def test_J_sanitze_packet_size(self):
+    def test_sanitze_packet_size(self):
         """
         verify that we conform to the rfc of packet and window sizes.
         """
@@ -821,7 +830,7 @@ class TransportTest(unittest.TestCase):
                              (2**32, MAX_WINDOW_SIZE)]:
             self.assertEqual(self.tc._sanitize_packet_size(val), correct)
 
-    def test_K_sanitze_window_size(self):
+    def test_sanitze_window_size(self):
         """
         verify that we conform to the rfc of packet and window sizes.
         """
@@ -831,9 +840,9 @@ class TransportTest(unittest.TestCase):
             self.assertEqual(self.tc._sanitize_window_size(val), correct)
 
     @slow
-    def test_L_handshake_timeout(self):
+    def test_handshake_timeout(self):
         """
-        verify that we can get a hanshake timeout.
+        verify that we can get a handshake timeout.
         """
         # Tweak client Transport instance's Packetizer instance so
         # its read_message() sleeps a bit. This helps prevent race conditions
@@ -864,7 +873,7 @@ class TransportTest(unittest.TestCase):
                           username='slowdive',
                           password='pygmalion')
 
-    def test_M_select_after_close(self):
+    def test_select_after_close(self):
         """
         verify that select works when a channel is already closed.
         """
@@ -926,7 +935,6 @@ class TransportTest(unittest.TestCase):
             chan.sendall(buffer(data))  # noqa: F821
             self.assertEqual(sfile.read(len(data)), data)
 
-    @needs_builtin('memoryview')
     def test_channel_send_memoryview(self):
         """
         verify sending memoryview instances to a channel
@@ -1051,3 +1059,25 @@ class TransportTest(unittest.TestCase):
         assert not self.ts.auth_handler.authenticated
         # Real fix's behavior
         self._expect_unimplemented()
+
+    def test_transport_channel_close(self):
+        """
+        verify concurrent channel closing while feeding it with data
+        """
+        self.setup_test_server()
+
+        def channel_close_thread(chan):
+            chan.close()
+
+        threads = []
+        for i in range(100):
+            chan = self.tc.open_session()
+            # trigger internal pipe creation
+            chan.fileno()
+            t = threading.Thread(target=channel_close_thread, args=(chan,))
+            threads.append(t)
+            t.start()
+            chan._feed(b'')
+
+        for t in threads:
+            t.join()
